'****************************************************************
'*  Name    : 16ch_last_u_d.BAS                                 *
'*  Author  : zuisti                                            *
'*  Date    : 2010.07.18  last_u_d             2030  34+70      *
'****************************************************************

'In Proteus we don't using line driver transistors, the outputs of the hc138 are active low, so
'the display lines are Common led Cathodes, the rows (driving with the hc595s) are led anodes 

        Device = 16F628A 

#define CCATHODE 1 ' for Proteus ISIS only !!!
'#DEFINE CCATHODE 0 ' for the real circuit !!! UNCOMMENT  also this config line !!!!!

'        Config INTRC_OSC_NOCLKOUT,WDT_OFF,PWRTE_ON,BODEN_OFF,LVP_OFF,CP_OFF,MCLRE_OFF,DATA_CP_OFF


        Xtal = 4
        Optimiser_Level = 3

        #define DISPLAYWIDTH 16 '8

        Symbol MAX_ASCII  250
        Symbol BUFADDRESS   160 '0A0h in BANK1

        Dim LineBuf[10*7]    As Byte At BUFADDRESS 
        ' if 16char, 10/line, 0..9 = line 0, 10...19 = line 1, ...
        ' if 8 char,  5/line, 0..4 = line 0, 5...8 = line 1, ...

        Dim PattBuf[7]      As Byte ' actual char line patterns, or ascii values for char test

        Dim bitmap_index    As Word
        Dim idx             As Byte 

        Dim mode_fl         As Bit ' 0 = char scroll, 1 = pixel scroll
        Dim IT_Flag         As Bit
        Dim ScrollStop_fl   As Bit
        Dim NoPixGap_fl     As Bit ' 0 = pixel gap ON
        Dim NoPixGap_Savfl  As Bit
                
        Dim ScrollTime      As Byte ' speed value, smaller is faster
        Dim scrollcnt       As Byte
        Dim chscrollcnt     As Byte
        Dim PixCnt          As Byte

        Symbol AsciiCode    chscrollcnt        
        Symbol FirstChar    chscrollcnt        
        Symbol LastPos      scrollcnt        

        Dim CharCnt         As Byte
        Dim charnum         As Byte
        Dim linenum         As Byte
        Dim pattern         As Byte

        Dim tmp1            As Byte
        Dim ch              As Byte  
        Dim rch             As Byte 'received char 

        Symbol LINEPORT PORTA ' outputs for Line decoder: PORTA<2:0> ;RA0 = LSB
        Symbol LINES_OFF 7    ' decoder IC output 7 is "free", 0...6 are use

        ' Scrolling Mode switch input: PORTA.4 ;RA4   (0: char, 1: pixel scrolling)
        Symbol MODESWITCH PORTA.4

        Symbol CLOCK_PIN PORTB.0   ' for shift regs clocking in 
        Symbol DATA_PIN  PORTB.7   ' the data itself
        Symbol STORE_PIN PORTA.3   ' clock the shift reg's output storage

        Symbol PORT_SP_SWITCH PORTB 	 ' speed switch inputs: PORTB<5:3>
        Symbol MASK_SP_SWITCH %00111000  ' RB3 = SW_1 , sum value *8 !!

        Symbol NOPIXGAP_SW PORTB.6        ' pixel gap on if 0

E_RD_ch Macro addr   'ch = Eread addr
#if (Prm_1 == Num8)
    @ movlw  addr
#else
    @ movf addr,w
#endif   
    GoSub _E_RD_ch
Endm

E_WR Macro addr, value  'Ewrite addr, [value] 
#if (Prm_1 == Num8)
    @ movlw  addr
#else
    @ movf addr,w
#endif   
    @ bsf STATUS,5
    @ movwf EEADR
#if (Prm_2 == Num8)
    @ movlw  value
#else
    @ bcf STATUS,5
    @ movf value,w
#endif   
    GoSub _E_WR
Endm

    Declare FSR_CONTEXT_SAVE = OFF

    On_Hardware_Interrupt GoTo SerialIT ' receive only !

' ***************** Program starts here

    CMCON = %00000111 ' switch off comparators
    TRISA = %10110000 ' 7, 5 and 4 PORTA pins are inputs

    ' Initialize USART module for receive (RX) only:
	TXSTA.2 = 1                'BRGH  
    SPBRG = 25                 '9600 Bd at 4 M clock
    RCSTA = %10010000          '8 bit, no parity

    ' empty rx fifo
    movf RCREG, w
    movf RCREG, w

    TRISB = %01111110                               ' only PORTB.0 and PORTB.7 are outputs
    STORE_PIN = 0 : CLOCK_PIN = 0 : DATA_PIN = 0    ' setup for positive clock pulses
    LINEPORT = LINES_OFF 'disable lines

    GoSub RestoreSettings

    IT_Flag = 0
    INTCON = $C0 : PIE1 = $20 'enable global ITs and RX it only

Restart:
    ScrollStop_fl = 0
    GoSub LoadBuf_Clear
    If CharCnt = 0 Then GoTo Storing' if the EEPROM is empty at start
    scrollcnt = 0            
    chscrollcnt = 0            
    While 1=1  ' main loop (forever)
        If IT_Flag = 1 Then
            ch = rch
            IT_Flag = 0
            If ch = 27 Then GoTo Storing
            If ch = ("t" - "a" + 1) Then GoTo CharTest 'control t
            If ch = ("s" - "a" + 1) Then If PixCnt <> 0 Then GoTo StepRoutine
            GoSub OnlineCommand
        End If
        If ScrollStop_fl = 0 Then If PixCnt <> 0 Then GoSub Scroll       
        GoSub DisplayBuf
    Wend  ' end of main loop

' ************ subroutines 

StepRoutine:
        NoPixGap_Savfl = 0
        If NoPixGap_fl = 1 Then NoPixGap_Savfl = 1
        NoPixGap_fl = 1
#if (DISPLAYWIDTH == 16)
        FirstChar = CharCnt - 17
#else
        FirstChar = CharCnt - 9
#endif
        If FirstChar > 127 Then FirstChar = 0
        GoSub ClearBuf
        GoTo StepStart
         
        While 1=1
            If IT_Flag = 1 Then
                ch = rch
                IT_Flag = 0
                If ch = 27 Then Break ' exit
                If ch = "<" Then If CharCnt < 127 Then 
                    Inc FirstChar
                    charnum = 1
                    GoTo Stepchar_loop
                End If
                If ch = ">" Then If FirstChar <> 0 Then 
                    Dec FirstChar
StepStart:
                    CharCnt = FirstChar
#if (DISPLAYWIDTH == 16)
                    charnum = 16
#else
                    charnum = 8
#endif
Stepchar_loop:
                        E_RD_ch CharCnt
                        Inc CharCnt
                        If ch = 255 Then  
                            CharCnt = 127
                            GoTo Text_end
                        End If
                        GoSub PutChar
                        decfsz charnum
                    GoTo Stepchar_loop
                End If
            End If            
Text_end:
            GoSub DisplayBuf
        Wend
        If NoPixGap_Savfl = 0 Then NoPixGap_fl = 0
    GoTo Restart

CharTest:
        NoPixGap_Savfl = 0
        If NoPixGap_fl = 1 Then NoPixGap_Savfl = 1
        NoPixGap_fl = 0 
        GoSub ClearBuf

        PattBuf[0] = " " ' space (32dec)
        PattBuf[1] = "x" ' 120dec
        PattBuf[2] = " "
        PattBuf[3] = " "
        PattBuf[4] = "1"
        PattBuf[5] = "2"
        PattBuf[6] = "0"
        AsciiCode = 120
        GoTo ChTest_Start

        While 1=1
            If IT_Flag = 1 Then
                ch = rch
                IT_Flag = 0
                If ch = 27 Then Break ' exit
                If ch = "+" Then 'up = incr
                    If AsciiCode < MAX_ASCII Then                 
                        Inc AsciiCode
                        If  AsciiCode = 128 Then
                            AsciiCode = 182 ' remapped 182 -> 128
                            movlw 128
                            movwf PattBuf#1
                            PattBuf[5] = "8"
                            PattBuf[6] = "2"
                        Else
                            incf PattBuf#1 ;Inc PattBuf[1]
                            incf PattBuf#6 ;Inc PattBuf[6]
                            If PattBuf[6] > "9" Then
                                PattBuf[6] = "0"
                                incf PattBuf#5 ;Inc PattBuf[5]                        
                                If PattBuf[5] > "9" Then
                                    PattBuf[5] = "0"
                                    incf PattBuf#4 ;Inc PattBuf[4]
                                End If
                            End If
                        End If    
                    End If
                Else If ch = "-" Then 
                    If AsciiCode > 33 Then 'down = decr 
                        Dec AsciiCode
                        If  AsciiCode = 181 Then
                            AsciiCode = 127
                            movlw 127
                            movwf PattBuf#1
                            PattBuf[5] = "2"
                            PattBuf[6] = "7"
                        Else
                            decf PattBuf#1 ;Dec PattBuf[1]
                            decf PattBuf#6 ;Dec PattBuf[6]
                            If PattBuf[6] < "0" Then
                                PattBuf[6] = "9"
                                decf PattBuf#5 ;Dec PattBuf[5]                        
                                If PattBuf[5] < "0" Then
                                    PattBuf[5] = "9"
                                    decf PattBuf#4 ;Dec PattBuf[4]
                                End If
                            End If
                        End If
                    End If
                End If
ChTest_Start:
                CharCnt = 0
                Repeat
                    ch = PattBuf[CharCnt]
                    GoSub PutChar
                    Inc CharCnt
                Until CharCnt = 7
            End If            
            GoSub DisplayBuf
        Wend
        NoPixGap_fl = 0 
        If NoPixGap_Savfl = 1 Then NoPixGap_fl = 1
    GoTo Restart

Storing:
        charnum = 0
        Repeat
            E_RD_ch charnum
            If ch <> 255 Then
                Inc charnum
            Else                
                linenum = charnum
                While linenum < 126
                    Inc linenum
                    E_RD_ch linenum
                    If ch <> 255 Then E_WR linenum, 255                                        
                Wend            
                Break
            End If
        Until charnum = 127
        LastPos = charnum
                    
        GoSub ClearBuf ' CharCnt = 0

Storing1:
        ch = 32
        GoSub PutChar
        ch = 32
        GoSub PutChar
        ch = "S" 
        GoSub PutChar
        ch = "t" 
        GoSub PutChar
        ch = "o" 
        GoSub PutChar
        ch = "r" 
        GoSub PutChar
        ch = "e" 
        GoSub PutChar
        ch = ":" 
        GoSub PutChar
    
        If CharCnt <> 0 Then ' was a backspace 
            charnum = 0
Storing2:
            Repeat
                E_RD_ch charnum
                GoSub PutChar
                Inc charnum
            Until charnum = CharCnt    
            GoTo Storing3
        End If

        While 1=1  
            If IT_Flag = 1 Then
                If rch = 27 Then If CharCnt > 0 Then Break ' ESC, new store again (forget previous) 
                ch = rch
                IT_Flag = 0
                If ch = 13 Then Break ' ENTER, end of storing, restart displaying
                If ch = 9 Then If LastPos <> 0 Then 'TAB, append, only at first and once
                    CharCnt = LastPos
                    GoTo Append
                End If

                If ch = 8 Then If CharCnt <> 0 Then  ' backspace
                    Dec CharCnt
Append:
                    LastPos = 0 ' no more append
#if (DISPLAYWIDTH == 16)
                    If CharCnt < 16 Then 
                        charnum = CharCnt
                        GoSub ClearBuf
                        CharCnt = charnum
                        GoTo Storing1
                    End If
                    charnum = CharCnt - 16
#else
                    If CharCnt < 8 Then GoTo Storing1
                    charnum = CharCnt - 8
#endif
                    GoTo Storing2
                End If        
                If ch > 31 Then 
                    LastPos = 0 ' no more append
                    If CharCnt < 127 Then 
                        If ch > $7F Then    'after the DEL (127dec) char
                            ch = ch - 54 ' 182-128  ' remap it to optimize for size
                            If ch < (182 - 54) Then ch = "?"
                            If ch > (250 - 54) Then ch = "?"
                        EndIf 
                        E_WR CharCnt, ch 
                        GoSub PutChar
                        Inc CharCnt
Storing3:
                        If CharCnt < 127 Then E_WR CharCnt, 255
'full begin
                   Else
                        GoSub ClearBuf
                        ch = "*"
                        GoSub PutChar
                        ch = 32
                        GoSub PutChar
                        ch = "F" 
                        GoSub PutChar
                        ch = "u" 
                        GoSub PutChar
                        ch = "l" 
                        GoSub PutChar
                        ch = "l" 
                        GoSub PutChar
                        ch = 32 
                        GoSub PutChar
                        ch = "*" 
                        GoSub PutChar
                        While 1=1
                            If IT_Flag = 1 Then
                                IT_Flag = 0
                                CharCnt = 127
                                GoTo Append ' Break ' any key for exit , back to store-append
                            End If
                            GoSub DisplayBuf
                            'delay for on ?
                        Wend
                                    
'full end
                    End If
'16ch_last_u_d begin              
                ElseIf ch = ("a" - "a" + 1) Or ch = ("s" - "a" + 1) Then
                        'ctrl a, prev. ASCII value   ctrl s, next value
                    tmp1 = ch
                    If CharCnt = 0 Then
                        ch = "A"
                    Else
                        Dec CharCnt
                        E_RD_ch CharCnt
                        If tmp1 <> 1 Then ' ctrl s, incr
                            If ch < (250 - 54) Then Inc ch 'remapping limit
                        ElseIf ch > 32 Then
                            Dec ch
                        EndIf
                    EndIf
                    E_WR CharCnt, ch 

                        Inc CharCnt

                    GoTo Append
'16ch_last_u_d end
                End If ' ch > 31 or ctrl a, ctrl s
            End If ' it
            GoSub DisplayBuf
        Wend 
    GoTo Restart

DisplayBuf:
        linenum = 7 ' line loop (we have 7 lines)
' ch as an index points now to the first byte of the unused 8th line in linebuf
#if (DISPLAYWIDTH == 16)
        ch = 7*10 '= 70 or 35
#else
        ch = 7*5
#endif
DispLine_loop:
            'ch = ch - 10(5) : charnum = 10(5) : idx = ch 'charnum and idx  are for char loop
#if (DISPLAYWIDTH == 16)
            movlw 10 ;10 or 5
#else
            movlw 5
#endif
            'charnum = 10(5) for ' 10(5) shift regs, 10(5)*8 = 80(40) columns
            movwf charnum ; counter for char loop
            subwf ch,w ; w = ch - 10
            movwf ch   ; points to the first byte of the actual line in linebuf
            movwf idx  ; temp index for char loop, now equal to ch
            
DispChar_loop:
                If IT_Flag = 1 Then Return 'to fast IT response
                pattern = LineBuf[idx]                
                tmp1 = 8 ' Shift out the whole byte
ShLoop:        
                rlf pattern
                btfsc STATUS.0
                    bsf DATA_PIN
                bsf CLOCK_PIN
                bcf CLOCK_PIN
                bcf DATA_PIN
                decfsz tmp1
                    GoTo ShLoop                        
                Inc idx 'next line-byte                
                decfsz charnum
            GoTo DispChar_loop
                
            ' the entire line is ready in the shift regs, now display it
            LINEPORT = LINES_OFF           ' disable old line
            STORE_PIN = 1 : STORE_PIN = 0  ' store  pulse for shift regs output storage 
            LINEPORT = linenum - 1         ' switch on the actual line driver

#if (DISPLAYWIDTH == 16)
            tmp1 = 15 'delay for better reading
#else
            tmp1 = 50
#endif

Dloop:
                If IT_Flag = 1 Then Return
                decfsz tmp1 ; dec tmp1 and skip if zero
            GoTo Dloop            
'            delayms 2

            decfsz linenum
        GoTo DispLine_loop
        ' end of line loop
    Return               

PattShift:
'       tmp1 = linenum * 10(5) + (BUFADDRESS-1) ' address of the line (1..7) in array linebuf 
        tmp1 = linenum << 1     '*2
        rlf tmp1                ;*4
        tmp1 = tmp1 + linenum   '*5
#if (DISPLAYWIDTH == 16)
        rlf tmp1                ;*10
#endif
        movlw (BUFADDRESS-1) ;159
        addwf tmp1,F
        'tmp1 = points to the last byte of the line  
        
PattShift1:
        FSR = tmp1  ' for indirect addressing (to last byte)
        rlf pattern ; shift left, carry to bit 0, bit 7 to carry
        rlf INDF    ; indirect shift left with carry the byte in linebuf
        Dec FSR     ; dec byte address in linebuf
        rlf INDF
        Dec FSR    
        rlf INDF
        Dec FSR    
        rlf INDF
#if (DISPLAYWIDTH == 16)
        Dec FSR    
        rlf INDF
        Dec FSR    
        rlf INDF
        Dec FSR    
        rlf INDF
        Dec FSR    
        rlf INDF
        Dec FSR    
        rlf INDF
#endif
        Dec FSR     ; first byte of the line 
        rlf INDF
    Return

PutChar:
        GoSub Make_index ' linenum = 0
        Repeat
            GoSub GetPattern ' also inc linenum and index
            GoSub PattShift
            GoSub PattShift1
            GoSub PattShift1
            GoSub PattShift1
            GoSub PattShift1
            If NoPixGap_fl = 0 Then GoSub PattShift1
        Until linenum = 7
    Return

Make_index:
        ' bitmap_index = (ch - 32) * 7 + (bitmap_address)
        movlw 32
        subwf ch,W
        movwf bitmap_index
        clrf BITMAP_INDEXH
        bcf STATUS,0
        rlf bitmap_index,F
        rlf BITMAP_INDEXH,F
        rlf bitmap_index,F
        rlf BITMAP_INDEXH,F
        rlf bitmap_index,F
        rlf BITMAP_INDEXH,F  ; *8

        subwf bitmap_index,F    ; - W = *7 ( w = ch-32 ) 
        btfss STATUS,0          ;if borrow, C=0 !!!
        decf BITMAP_INDEXH,F

        movlw (bitmap & 255)
        addwf bitmap_index,F
        movlw ((bitmap >> 8) & 255)
        btfsc STATUS,0
        addlw 1
        addwf BITMAP_INDEXH,F

        linenum = 0
    Return

GetPattern:
        'pattern = LRead8 bitmap[bitmap_index]
        GoSub GetTableItem
        movwf pattern

        Inc bitmap_index
        Inc linenum
#if CCATHODE == 1
        pattern = ~pattern
#endif
        rlf pattern
        rlf pattern
        If NoPixGap_fl = 1 Then rlf pattern
    Return

GetTableItem:
        movf BITMAP_INDEXH,W
        movwf PCLATH
        movf bitmap_index,W
    movwf PCL ; 'goto to the proper retlw instr, returns with pattern in w reg

ClearBuf:
#if (DISPLAYWIDTH == 16)
        CharCnt = 7*10 '= 70 or 35
#else
        CharCnt = 7*5
#endif
Cbuf_loop:
#if CCATHODE == 1
            LineBuf[CharCnt - 1] = 0
#else
            LineBuf[CharCnt - 1] = 255
#endif
            decfsz CharCnt
        GoTo Cbuf_loop
    Return

LoadBuf_Clear:
        GoSub ClearBuf ' CharCnt = 0
LoadBuf:
#if (DISPLAYWIDTH == 16) 
        charnum = CharCnt + 13
        If NoPixGap_fl = 1 Then charnum = charnum + 3
#else
        charnum = CharCnt + 6
        If NoPixGap_fl = 1 Then charnum = charnum + 2
#endif
        Repeat
            E_RD_ch CharCnt
            If ch = 255 Then Break 'eeprom text end
            GoSub PutChar
            Inc CharCnt
        Until CharCnt = charnum
        PixCnt = 0
        If CharCnt = charnum Then GoSub NextChar
   Return

_E_RD_ch:
        bsf STATUS,5
        movwf EEADR
        bsf EECON1,RD
        movf EEDATA,W
        bcf STATUS,5
        movwf ch
	 Return

Scroll:
        Inc scrollcnt
        If scrollcnt < ScrollTime Then Return
        scrollcnt = 0
        If mode_fl = 1 Then ' pixel scroll            
            If PixCnt = 1 Then GoSub NextChar
            Dec PixCnt
        Else 'char scroll
            Inc chscrollcnt
            If chscrollcnt < 5 Then Return
            chscrollcnt = 0
        End If

        linenum = 7
Scr_lineloop:
            pattern = PattBuf[linenum - 1]
            GoSub PattShift
            If mode_fl = 1 Then
                PattBuf[linenum - 1] = pattern
            Else
                GoSub PattShift1
                GoSub PattShift1
                GoSub PattShift1
                GoSub PattShift1
                If NoPixGap_fl = 0 Then GoSub PattShift1
            End If
            decfsz linenum
        GoTo Scr_lineloop 

        If mode_fl = 1 Then Return 'pixel
NextChar:
        E_RD_ch CharCnt
        If ch = 255 Then 
            CharCnt = 0
            GoTo NextChar 'back to begin
        End If
        GoSub Make_index  ' linenum = 0
        Repeat
            GoSub GetPattern  ' Inc linenum
            PattBuf[linenum - 1] = pattern
        Until linenum = 7
        Inc CharCnt
        If CharCnt = 127 Then CharCnt = 0
        PixCnt = 7
        If NoPixGap_fl = 1 Then Dec PixCnt ' = 6
    Return

SerialIT:
        movf RCREG, w  ' get the received char
'        rch = RCREG
        If IT_Flag = 0 Then 
            movwf rch
            IT_Flag = 1
        End If
    Context Restore ' return from interrupt with restore regs

OnlineCommand:
    Branch ch,[L_0,L_1,L_2,L_3,L_4,L_5,L_6,L_7,L_8,L_9,L_10,L_11,L_12,L_13,L_14] '_
               'L_15,L_16,L_17,L_18,L_19,L_20,L_21,L_22,L_23,L_24,L_25,L_26,L_27]

L_0:  'ch = 0 (null) or ch > 14 '27 
L_13: 'ctrl m, enter, not used here

'L_15: 'ctrl o
'L_16: 'ctrl p
'L_17: 'ctrl q
'L_18: 'ctrl r
'L_19: 'ctrl s
'L_20: 'ctrl t, char test, not used here
'L_21: 'ctrl u
'L_22: 'ctrl v
'L_23: 'ctrl w
'L_24: 'ctrl x
'L_25: 'ctrl y
'L_26: 'ctrl z
'L_27: 'escape
    Return

L_1: 'ctrl a, pause scroll
        ScrollStop_fl = 1
    Return

L_2: 'ctrl b, continue scroll
        ScrollStop_fl = 0
    Return

L_3: 'ctrl c, char scroll
        mode_fl = 0
    Return

L_4: 'ctrl d, pixel scroll
        mode_fl = 1
    Return

L_5: 'ctrl e, scroll speed down
        If ScrollTime < 63 Then Inc ScrollTime
    Return

L_6: 'ctrl f, scroll speed up
        If ScrollTime > 1 Then Dec ScrollTime
    Return

L_8: 'ctrl h, backspace, no pixel gap
        NoPixGap_fl = 1
    GoTo _LB_Clear

L_9: 'ctrl i, tab, pixel gap on
        NoPixGap_fl = 0                         
_LB_Clear:
        If PixCnt <> 0 Then Return
L_7: 'ctrl g, display home
    GoTo LoadBuf_Clear
        

L_10: 'ctrl j, store settings to EEPROM
        ch = ScrollTime
        If mode_fl = 1 Then ch.7 = 1 'pixel scr            
        If NoPixGap_fl = 1 Then ch.6 = 1
EW_127_ch:    
        'EWrite 127,[ch]
        movlw 127
        bsf STATUS,5
        movwf EEADR
        bcf STATUS,5
        movf ch,w 
_E_WR:
        bsf STATUS,5
        movwf EEDATA
        bsf EECON1,WREN
        movlw 85
        movwf EECON2
        movlw 170
        movwf EECON2
        bsf EECON1,WR
        btfsc EECON1,WR
        @ GoTo $ - 1
        bcf EECON1,WREN
        bcf STATUS,5
    Return
    
L_11: 'ctrl k, restore settings from EEPROM if any, or from switches
RestoreSettings:
        E_RD_ch 127 ' ch = Eread 127 (last eeprom byte)
        If ch = 255 Then GoTo GetSwitch
        ScrollTime = ch & 63           
        mode_fl = 1 ' pixel scroll
        If ch.7 = 0 Then mode_fl = 0 ' char                
        NoPixGap_fl = 0
        If ch.6 = 1 Then NoPixGap_fl = 1    
    Return    

L_12: 'ctrl l, clear stored settings in EEPROM
        ch = 255
    GoTo EW_127_ch    

L_14: 'ctrl n, get settings from switches
GetSwitch:
        mode_fl = 1 ' pixel scroll
        ScrollTime = (PORT_SP_SWITCH & MASK_SP_SWITCH)
        Inc ScrollTime
        If MODESWITCH = 0 Then  mode_fl = 0 ' char scroll   
        NoPixGap_fl = 0
        If NOPIXGAP_SW = 1 Then NoPixGap_fl = 1 
    Return

'**********
    Include "bitmap7.inc"    ' LDATA table 32-127, lenght = 300H
    Include "Greek7m.inc"    ' 182 - 250
' ******** end of program

